"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var IndexCache_1 = require("../utils/IndexCache");
var SizeAndPositionManager = /** @class */ (function () {
    function SizeAndPositionManager(count, size, indexCountToAdd, timesToAddFirstly) {
        if (indexCountToAdd === void 0) { indexCountToAdd = 1; }
        if (timesToAddFirstly === void 0) { timesToAddFirstly = 0; }
        this._fullSize = 0;
        this._lastCalculatedIndex = -1;
        if (count < 0) {
            throw new Error('"count" param must me > 0');
        }
        if (indexCountToAdd < 1) {
            throw new Error('"indexCountToAdd" param must be >= 1');
        }
        this._count = count;
        this._indexCountToAdd = indexCountToAdd;
        this._size = size;
        if (typeof size !== 'number') {
            this._pixelCache = new Map();
            this._indexCache = new IndexCache_1.default();
        }
        if (isFinite(count)) {
            count && this._recalculateFullSize(count - 1);
        }
        else if (timesToAddFirstly) {
            this._recalculateFullSize(indexCountToAdd * timesToAddFirstly);
        }
    }
    Object.defineProperty(SizeAndPositionManager.prototype, "count", {
        get: function () {
            return this._count;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SizeAndPositionManager.prototype, "fullSize", {
        get: function () {
            return this._fullSize;
        },
        enumerable: true,
        configurable: true
    });
    SizeAndPositionManager.prototype._getIndex = function (index) {
        if (index < 0 || index >= this.count) {
            debugger;
            throw new Error("Index " + index + " is out of bounds");
        }
        return index;
    };
    SizeAndPositionManager.prototype._recalculateFullSize = function (index) {
        if (!isFinite(this.count)) {
            index = Math.floor(index / this._indexCountToAdd) * this._indexCountToAdd;
        }
        if (this._lastCalculatedIndex < index) {
            if (typeof this._size === 'number') {
                this._fullSize = (index + 1) * this._size;
            }
            else {
                for (var i = this._lastCalculatedIndex + 1; i <= index; i++) {
                    this._fullSize += this.getSize(i);
                }
            }
            this._lastCalculatedIndex = index;
        }
    };
    SizeAndPositionManager.prototype.getSize = function (index) {
        index = this._getIndex(index);
        return typeof this._size === 'number' ? this._size : this._size(index);
    };
    SizeAndPositionManager.prototype._getPixelByIndex = function (index, sum) {
        if (sum === void 0) { sum = 0; }
        if (!this._pixelCache.has(index)) {
            var pixel = index > 0 ? this._getPixelByIndex(index - 1, this.getSize(index - 1)) : 0;
            this._pixelCache.set(index, pixel);
        }
        return sum + this._pixelCache.get(index);
    };
    SizeAndPositionManager.prototype.getPixelByIndex = function (index) {
        index = this._getIndex(index);
        var pixel = 0;
        if (typeof this._size === 'number') {
            pixel = index * this._size;
        }
        else {
            pixel = this._getPixelByIndex(index);
        }
        this._recalculateFullSize(index);
        return pixel;
    };
    SizeAndPositionManager.prototype.getIndexByPixel = function (pixel) {
        if (pixel < 0 || !this.count || (isFinite(this.count) && pixel > this.fullSize)) {
            throw new Error('Pixel is out of bounds');
        }
        var index;
        if (typeof this._size === 'number') {
            index = Math.min(Math.floor(pixel / this._size), this.count - 1);
        }
        else {
            index = this._indexCache.get(pixel);
            if (index === -1) {
                var count = Math.max(this._lastCalculatedIndex, 0);
                index = count * Math.floor(pixel / (this.fullSize || 1));
                while (true) {
                    var start = this.getPixelByIndex(index);
                    var end = start + this.getSize(index);
                    if (!this._indexCache.has(start)) {
                        this._indexCache.set({ start: start, end: end, index: index });
                    }
                    if (pixel < start) {
                        index--;
                    }
                    else if (pixel >= end && index !== this.count - 1) {
                        index++;
                    }
                    else {
                        break;
                    }
                }
            }
        }
        this._recalculateFullSize(index);
        return index;
    };
    return SizeAndPositionManager;
}());
exports.default = SizeAndPositionManager;
